local _, L = ...;

function MiraMod_OnLoad()

	-- Register events
	MiraMod:RegisterEvent("VARIABLES_LOADED");
	MiraMod:RegisterEvent("RAID_TARGET_UPDATE");
	MiraMod:RegisterEvent("CHAT_MSG_MONSTER_YELL");
	MiraMod:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	MiraMod:RegisterEvent("PLAYER_ENTERING_WORLD");
	MiraMod:RegisterEvent("ENCOUNTER_START");
	
	SlashCmdList["MiraMod"] = MiraMod_OnCommand;
	SLASH_MiraMod1 = "/Miramod";
	SLASH_MiraMod2 = "/miramod";
	
	-- Init global variables
	UpdateTimeInterval 				= 0.5;
	TimeSinceLastUpdate				= 0;
	MiraModTextDisplay 				= nil;
	SoundDir 						= nil;
	
	InstancesList					= {};	-- List of instance names for which we have a sub addon
	InstancesActive					= {};	-- true, if that sub addon is active
	InstancesInitFunction			= {};	-- Function that activates that sub addon
		
	-- Debuff-Scanner	
	DebuffListNames					= {};	-- List of names of the debuff
	DebuffStates					= {};	-- State of the debuff true = active, false = not active
	DebuffListAddSound				= {};	-- Played soundfile when debuff is detected
	DebuffListAddTextMsg			= {};  	-- Displayed text message
	DebuffListRemoveSound			= {};	-- Played soundfile when debuff is removed
	DebuffListRemoveTextMsg			= {};  	-- Displayed text message
	DebuffScanInGroup				= {}; 	-- True if this debuff has to be searched in the group, to find special debuffs like blackout
	DebuffYellAddText				= {};	-- Text which is yelled when debuff is detected
	DebuffYellRemoveText			= {};	-- Text which is yelled when debuff is removed
	DebuffWarnStackCount			= {};	-- Warning will be only fired when the debuff reaches this or more stack count
	DebuffTiggersCountdown			= {};	-- True, if this debuffs causes an countdown
	
	DebuffCountdown					= {}; 	-- Used for countdown
	DebuffCountdownPos				= {};	-- Position for countdown
	
	-- Combatlog Scanner
	CombatSource					= {};	-- Sourcename
	CombatActionType				= {};	-- can be SPELL_CAST_SUCCESS, SPELL_AURA_APPLIED, SPELL_AURA_REMOVED
	CombatSpellId					= {};	-- Id of the spell, if knowen
	CombatSpellName					= {};	-- Name of the spell
	CombatSound						= {};	-- Sound which is played when event is fired
	CombatTextMsg					= {};	-- Text messages which is displayed
	CombatFunction					= {};	-- Function to be executed (first parameter of the called function is target name)
	CombatStackCount				= {};	-- Stackcount
	CombatTarget					= {};	-- Target of the spell
	
	YellCheckFunctions				= {};
	RaidTargetFunctions				= {};
	EncounterStartFunctions			= {};	
		
	-- Special timers	
	SpecialTimersEnd				= {};
	SpecialTimersMsg				= {};
	SpecialTimersSnd				= {};
	SpecialTimersFunction			= {};
	
	nextraid_symbol 				= 8;
		
end

function MiraMod_PlaySound(file)

	PlaySoundFile(SoundDir .. file, "Master");

end

function MiraMod_GetTranslation(id)

	if(id == nil) then
	
		return "-";
	
	end 

	return L[id];

end

function MiraMod_OnCommand(arg)

	local str = L[arg];
	DEFAULT_CHAT_FRAME:AddMessage("L["..arg.."] = "..str);
	
	if(arg=="combat") then
	
		local CombatCount = table.getn(CombatSource);
		
		for za=1,CombatCount do
		
			if(CombatSource[za] ~= nil and CombatSpellName[za] ~= nil) then
			
				local src = "N/A";
				if(CombatSource[za] ~= nil) then
					src = CombatSource[za];
				end
					
				local spell = "N/A";
				if(CombatSpellName[za] ~= nil) then 
					spell = CombatSpellName[za];
				end
				
				local dest = "N/A";
				if(CombatTarget[za] ~= nil) then
					dest = CombatTarget[za];
				end				
				
				local msg = "CombatSource["..za.."] = "..src..", Spell["..za.."] = "..spell..", Target: "..dest;
				DEFAULT_CHAT_FRAME:AddMessage(msg);

			end
		
		end
	end
	
	if(arg=="debuff") then
	
		local DebuffCount = table.getn(DebuffListNames);
		
		for za=1,DebuffCount do
		
			if(DebuffListNames[za] ~= nil) then
		
				local msg = "DebuffListNames["..za.."] = "..DebuffListNames[za];
				DEFAULT_CHAT_FRAME:AddMessage(msg);
			
			else

				local msg = "DebuffListNames["..za.."] = is nil";
				DEFAULT_CHAT_FRAME:AddMessage(msg);
			
			end
		
		end	
	
	end

end

function MiraMod_OnVariablesLoaded()

	MiraModTextDisplay = _G["MiraModTextDisplay"];

	MiraModTextDisplay:Show();
	MiraModTextDisplay:SetFadeDuration(1);
	MiraModTextDisplay:SetTimeVisible(2);
	
	if(SoundDir == nil) then
		SoundDir = "Interface\\AddOns\\MiraMod\\Sounds\\en";
	end
		
	SoundDir = SoundDir .. "\\";
	
	MiraMod_AddInstance("BlackwingDescent", MiraMod_BlackwingDescent_Init);
	MiraMod_AddInstance("TheBastionofTwilight", MiraMod_TheBastionofTwilight_Init);
	MiraMod_AddInstance("IcecrownCitadel", MiraMod_IcecrownCitadel_Init);
	MiraMod_AddInstance("Firelands", MiraMod_Firelands_Init);
	MiraMod_AddInstance("MogushanVaults", MiraMod_MogushanVaults_Init);
	MiraMod_AddInstance("ThroneoftheFourWinds", MiraMod_FourWinds_Init);
	MiraMod_AddInstance("HeartofFear", MiraMod_HeartofFear_Init);
	MiraMod_AddInstance("DragonSoul", MiraMod_DragonSoul_Init);
	MiraMod_AddInstance("TheRubySanctum", MiraMod_RubySanctum_Init);
	MiraMod_AddInstance("SiegeofOrgrimmar", MiraMod_SiegeofOrgrimmar_Init);
		
	-- Test
	-- MiraMod_AddDebuff("Weakened Soul", "seuche.mp3", "Weakened Soul Get", "verbindungweg.mp3", "Weakened Soul Rem", false, "Chogall_Corruption_Sickness_Yell_Add", nil, 0, false, 13);			
	-- MiraMod_AddCombat("Dorijana", "SPELL_AURA_APPLIED", 0, "Power Word Fortitude", "lauf.mp3", "Power Word Fortitude", MiraMod_SetRaidSymbol);
	
	-- Minor instances
	
	-- Evolution (Corla, Botin des Zwielichts, Schwarzfelshhlen)
	MiraMod_AddDebuff("Corla_Evolution", nil, nil, "verbindungweg.mp3", "Corla_Evolution_Rem");
							
	-- Kugelgefngnis (Hallen des Urspungs Trash)
	MiraMod_AddDebuff("BubbleBound", "seuche.mp3", "BubbleBound_Add", "verbindungweg.mp3", "BubbleBound_Rem", false, "BubbleBound_Yell_Add", "BubbleBound_Yell_Rem");
		
	-- Schattenvortex (Burg Schattenfang Trash)
	MiraMod_AddDebuff("Shadowvortex", "lauf.mp3", "Shadowvortex_Add", "verbindungweg.mp3", "Shadowvortex_Rem");	
	
	-- Statische Aufladung (Asaad, Vortex Gipfel)
	MiraMod_AddCombat("Asaad", "SPELL_CAST_START", 0, "Asaad_Static_Cling", "seuche.mp3", "Asaad_Static_Cling_Msg");
	
	MiraMod_AddYellCheckFunction(MiraMod_MinorYell);
		
	-- Say hello		
	if(DEFAULT_CHAT_FRAME ~= nil) then
		DEFAULT_CHAT_FRAME:AddMessage(L["StartUpMsg"], 0.5, 0.5, 1.0);
	else
		MiraModTextDisplay:AddMessage(L["StartUpMsg"], 0.5, 0.5, 1.0);	
	end
	
end

function MiraMod_MinorYell(arg1, arg2)

	-- Cata
	if(arg2 == L["Ozruk"]) then
	
		if(arg1 == L["Ozruk_Reflect"]) then
			MiraMod_PlaySound("seuche.mp3");
			MiraModTextDisplay:AddMessage(L["Ozruk_Reflect_Msg"], 1.0, 0.32, 0.66);
		end
		
	end

end

function MiraMod_AddInstance(name, initFunction)

	table.insert(InstancesList,   L[name]);
	table.insert(InstancesActive, false);
	table.insert(InstancesInitFunction, initFunction);

end

function MiraMod_TableInsert(atable, atext)

	table.insert(atable, MiraMod_GetTranslation(atext));

end

function MiraMod_TableInsertNoTranslation(atable, atext)

	if(atext == nil) then
	
	atext = "-";
	
	end

	table.insert(atable, atext);

end

function MiraMod_AddDebuff(name, addSound, addTextMsg, removeSound, removeMsg, scanInGroup, addYell, removeYell, stackCount, tiggerCountdown, countDown)

	MiraMod_TableInsert					(DebuffListNames, 			name);
	MiraMod_TableInsertNoTranslation	(DebuffListAddSound, 		addSound);
	MiraMod_TableInsert					(DebuffListAddTextMsg, 		addTextMsg);
	MiraMod_TableInsertNoTranslation	(DebuffListRemoveSound, 	removeSound);
	MiraMod_TableInsert					(DebuffListRemoveTextMsg,	removeMsg);
		
	if(scanInGroup == nil) then
		scanInGroup = false;
	end
	
	table.insert(DebuffScanInGroup, scanInGroup);
	
	MiraMod_TableInsert					(DebuffYellAddText,			addYell);
	MiraMod_TableInsert					(DebuffYellRemoveText,		removeYell);
		
	if(stackCount == nil) then
		stackCount = 0;
	end
			
	table.insert(DebuffWarnStackCount, stackCount);
	
	if(tiggerCountdown == nil) then
		tiggerCountdown = false;
	end
	
	table.insert(DebuffTiggersCountdown, tiggerCountdown);
	
	if(countDown == nil) then
		countDown = 0;
	end
	
	table.insert(DebuffCountdown, countDown);
	
	table.insert(DebuffStates, false);
	table.insert(DebuffCountdownPos, 0);

end

function MiraMod_AddCombat(source, actionType, spellId, spellName, sound, textMsg, func, stackCount, target)

	MiraMod_TableInsert					(CombatSource,			source);
	MiraMod_TableInsertNoTranslation	(CombatActionType,		actionType);
	MiraMod_TableInsertNoTranslation	(CombatSpellId,			spellId);
	MiraMod_TableInsert					(CombatSpellName,		spellName);
	MiraMod_TableInsertNoTranslation	(CombatSound,			sound);
	MiraMod_TableInsert					(CombatTextMsg,			textMsg);
	MiraMod_TableInsert					(CombatFunction,		func);
	
	if(stackCount == nil) then
		stackCount = 0;
	end
	
	table.insert(CombatStackCount, stackCount);
	
	MiraMod_TableInsert					(CombatTarget,			target);
	
end

function MiraMod_AddSpecialTimer(EndTime, Msg, Snd, Function)
	
	table.insert(SpecialTimersEnd, 		EndTime);
	table.insert(SpecialTimersMsg, 		MiraMod_CheckParam(Msg));
	table.insert(SpecialTimersSnd, 		MiraMod_CheckParam(Snd));
	table.insert(SpecialTimersFunction, MiraMod_CheckParam(Function));

end

function MiraMod_CheckParam(Param)

	if(Param == nil) then
		return "-";
	end	
	
	return Param;

end

function MiraMod_AddYellCheckFunction(yellFunc)

	table.insert(YellCheckFunctions, yellFunc);

end

function MiraMod_AddRaidTargetUpdateFunction(raidFunc)

	table.insert(RaidTargetFunctions, raidFunc);

end

function MiraMod_AddEncounterStartFunction(encounterFunc)

	table.insert(EncounterStartFunctions, encounterFunc);
	
end

function MiraMod_SetRaidSymbol(Target)

	if(Target ~= nil) then
		
		local icon = GetRaidTargetIndex(Target);
		
		if(icon ~= nextraid_symbol) then 
			SetRaidTargetIcon(Target, nextraid_symbol);
		end
		
		if(nextraid_symbol == 8) then
		
			table.insert(SpecialTimersEnd, time() + 1);
			table.insert(SpecialTimersMsg, nil);
			table.insert(SpecialTimersSnd, nil);
			table.insert(SpecialTimersFunction, MiraMod_OnResetRaidSymbol);		
		
		end
	end 
	
	nextraid_symbol = nextraid_symbol - 1;

	if(nextraid_symbol<0) then
		nextraid_symbol = 8;
	end

end

function MiraMod_RemoveRaidSymbol(Target)

	if(Target ~= nil) then
	
		local icon = GetRaidTargetIndex(Target);
		if(icon ~= nil) then
		
			SetRaidTargetIcon(Target, icon);
		
		end
	
	end

end

function MiraMod_OnResetRaidSymbol()

	nextraid_symbol = 8;

end

function MiraMod_OnUpdate(self, elapsed)

	if(MiraModTextDisplay) then

		TimeSinceLastUpdate = TimeSinceLastUpdate + elapsed;

		if (TimeSinceLastUpdate > UpdateTimeInterval) then
		
			local numplayers = GetNumGroupMembers();
	
			local za = 0;
			local debuffCount = table.getn(DebuffListNames);
				
			for za=1, debuffCount do
										
				local Debuff, Rank, Icon, Count, DebuffType, Duration, expirationTime;
				
				if(DebuffScanInGroup[za] == false) then

					if(DebuffListNames[za] ~= nil) then
					
						Debuff, Rank, Icon, Count, DebuffType, Duration, expirationTime = UnitDebuff("player", DebuffListNames[za]);
					
						if(Debuff == nil) then
							Debuff, Rank, Icon, Count, DebuffType, Duration, expirationTime = UnitBuff("player", DebuffListNames[za]);
						end
							
					end	
				
				else
					
					for zb=1,numplayers do
						
						local name = GetRaidRosterInfo(zb);
						
						if(name ~= nil) then
							Debuff, Rank, Icon, Count = UnitDebuff(name, DebuffListNames[za]);
							if(Debuff ~= nil) then
								break
							end
						end
							
					end
					
				end
				
				if(Debuff) then
								
					if(not DebuffStates[za] and (DebuffWarnStackCount[za]==0 or Count >= DebuffWarnStackCount[za])) then

						if(DebuffListAddSound[za] ~= "-") then 
							MiraMod_PlaySound(DebuffListAddSound[za]);
						end
						
						if(DebuffListAddTextMsg[za] ~= "-") then
							MiraModTextDisplay:AddMessage(DebuffListAddTextMsg[za], 1.0, 0.32, 0.66);
						end
						
						if(DebuffYellAddText[za] ~= "-") then
							SendChatMessage(DebuffYellAddText[za], "YELL", nil, 0);
						end
					
						DebuffStates[za] = true;
						
						if(DebuffTiggersCountdown[za] == true) then
							DebuffCountdown[za]		= math.floor(expirationTime - GetTime()) - 1;
						end						

						if(DebuffCountdown[za] > 0) then
							DebuffCountdownPos[za] = DebuffCountdown[za];
						end
						
					end
				end
				
				
				if(DebuffCountdownPos[za] ~= nil) then
							
					if(DebuffCountdownPos[za]>UpdateTimeInterval) then
						
						if(DebuffCountdownPos[za] == 9) then
							MiraMod_PlaySound("9.mp3");
						end
							
						if(DebuffCountdownPos[za] == 8) then
							MiraMod_PlaySound("8.mp3");
						end

						if(DebuffCountdownPos[za] == 7) then
							MiraMod_PlaySound("7.mp3");
						end

						if(DebuffCountdownPos[za] == 6) then
							MiraMod_PlaySound("6.mp3");
						end
						
						if(DebuffCountdownPos[za] == 5) then
							MiraMod_PlaySound("5.mp3");
						end
						
						if(DebuffCountdownPos[za] == 4) then
							MiraMod_PlaySound("4.mp3");
						end

						if(DebuffCountdownPos[za] == 3) then
							MiraMod_PlaySound("3.mp3");
						end
						
						if(DebuffCountdownPos[za] == 2) then
							MiraMod_PlaySound("2.mp3");
						end

						if(DebuffCountdownPos[za] == 1) then
							MiraMod_PlaySound("1.mp3");
						end
						
						DebuffCountdownPos[za] = DebuffCountdownPos[za] - UpdateTimeInterval;
						
					end
				
				end
				
				if(not Debuff and DebuffStates[za]==true) then
				
					DebuffStates[za] = false;
				
					if(DebuffListRemoveTextMsg[za] ~= "-") then					
						MiraModTextDisplay:AddMessage(DebuffListRemoveTextMsg[za], 0.32, 1.00, 0.32);					
					end
					
					if(DebuffListRemoveSound[za] ~= "-") then
						MiraMod_PlaySound(DebuffListRemoveSound[za]);
					end
					
					if(DebuffYellRemoveText[za] ~= "-") then
						SendChatMessage(DebuffYellRemoveText[za], "YELL", nil, 0);
					end						
					
					if(DebuffCountdownPos[za]>0) then
						DebuffCountdownPos[za] = 0;
					end
					
				end
				
			end
				
			local TimerSize 	= #SpecialTimersEnd;
			local CurrentTime 	= time();
								
			for za=TimerSize,1,-1 do
			
				if(SpecialTimersEnd[za] ~= "-") then 
								
					if(SpecialTimersEnd[za] <= CurrentTime) then
					
						-- Timer ran out
						
						local Msg = SpecialTimersMsg[za];
						local Snd = SpecialTimersSnd[za];
						local Func= SpecialTimersFunction[za];
						
						if(Msg ~= "-") then

							MiraModTextDisplay:AddMessage(Msg, 1.0, 1.0, 0.55);
						
						end
						
						if(Snd ~= "-") then
						
							local FileName = SoundDir .. Snd;
							MiraMod_PlaySound(FileName);
						
						end
						
						if(Func ~= "-") then
						
							Func();
						
						end
						
						table.remove(SpecialTimersEnd, za);
						table.remove(SpecialTimersMsg, za);
						table.remove(SpecialTimersSnd, za);
						table.remove(SpecialTimersFunction, za);
					
					end

				end				
			
			end
			
			TimeSinceLastUpdate = 0;
		
		end

	end

end

function MiraMod_ZoneCheck()

	local inInstance = IsInInstance();
			
	if(inInstance) then
	
		local name = GetInstanceInfo();
		
		DEFAULT_CHAT_FRAME:AddMessage("[MiraMod] InstanceName: " .. name);
		
		local instancesLength = table.getn(InstancesList);
		
		for za=0, instancesLength do
		
			if(name == InstancesList[za] and InstancesActive[za] == false) then
			
				InstancesActive[za] = true;
				
				if(InstancesInitFunction[za] ~= nil) then
					InstancesInitFunction[za]();
				else
					DEFAULT_CHAT_FRAME:AddMessage("[MiraMod] can't activate " .. name);
				end
				
				break;
			
			end
		
		end
	
	end
		
end

function MiraMod_OnEvent(self, event, ...)

local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18 = select(1, ...)

	if event == "VARIABLES_LOADED" then
		MiraMod_OnVariablesLoaded();
	end

	if MiraModTextDisplay == nil then
		return;
	end
	
	if event == "PLAYER_ENTERING_WORLD" then
		MiraMod_ZoneCheck();
	end
	
	if event == "ENCOUNTER_START" then
			
		DEFAULT_CHAT_FRAME:AddMessage("[MiraMod] Start encounter: " .. arg2.. ", calling functions...");
	
		local EncounterStartCount = table.getn(EncounterStartFunctions);
	
		for za=1, EncounterStartCount do
				
			EncounterStartFunctions[za](arg1, arg2, arg3, arg4);
						
		end
		
	end
	
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
				
		local za = 0;
		local found = false;
		local CombatCount = table.getn(CombatSource);
						
		for za=1, CombatCount do
												
			if ((CombatSource[za] == arg5 and CombatActionType[za] == arg2 and (CombatSpellId[za] == arg12 or CombatSpellName[za] == arg13)) or
			   (CombatSource[za] == "" and CombatActionType[za] == arg2 and (CombatSpellId[za] == arg12 or CombatSpellName[za] == arg13)) or
			   (CombatActionType[za] == "UNIT_DIED" and CombatActionType[za] == arg2 and CombatSource[za] == arg9)) then
			   			
				if(CombatStackCount[za] == "-" or CombatStackCount[za] == arg16) then
												
					if(CombatTarget[za] == "-" or CombatTarget[za] == arg9) then
										
						found = true;
							
						if(CombatSound[za] ~= "-") then 
							MiraMod_PlaySound(CombatSound[za]);
						end
						
						if(CombatTextMsg[za] ~= "-") then
							MiraModTextDisplay:AddMessage(CombatTextMsg[za], 1.0, 0.32, 0.66);
						end
						
						if(CombatFunction[za] ~= "-") then									
							CombatFunction[za](arg9);
						end
						
						break;
					
					end
				
				end
			end
		end
		
	end
	
	if event == "CHAT_MSG_MONSTER_YELL" then	

		local yellFunctionsCount = table.getn(YellCheckFunctions);
	
		for za=1, yellFunctionsCount do
		
			YellCheckFunctions[za](arg1, arg2);
		
		end
	
	end
	
	if event == "RAID_TARGET_UPDATE" then
	
		local raidTargetFunctionsCount = table.getn(RaidTargetFunctions);
		
		for za=1, raidTargetFunctionsCount do
		
			RaidTargetFunctions[za]();
		
		end

	end
	
end